@@ -4,7 +4,7 @@ package JSON::Schema::Draft201909::Utilities;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Internal utilities for JSON::Schema::Draft201909
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 use 5.016;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -37,6 +37,7 @@ our @EXPORT_OK = qw(
   assert_keyword_type
   assert_pattern
   assert_uri_reference
+  assert_uri
   annotate_self
   true
   false
@@ -226,7 +227,7 @@ sub A {
 # errors (consider if we were in the middle of evaluating a "not" or "if")
 sub abort {
   my ($state, $error_string, @args) = @_;
-  E($state, 'EXCEPTION: '.$error_string, @args);
+  E($state, $error_string, @args);
   die pop @{$state->{errors}};
 }
 
@@ -238,7 +239,10 @@ sub assert_keyword_type {
 
 sub assert_pattern {
   my ($state, $pattern) = @_;
-  try { qr/$pattern/; }
+  try {
+    local $SIG{__WARN__} = sub { die @_ };
+    qr/$pattern/;
+  }
   catch ($e) { return E($state, $e); };
   return 1;
 }
@@ -246,13 +250,37 @@ sub assert_pattern {
 sub assert_uri_reference {
   my ($state, $schema) = @_;
 
-  # for now, we just check for fragment validity
   my $ref = $schema->{$state->{keyword}};
-  return 1 if $ref !~ /#/
-    or $ref =~ m{#$}                          # empty fragment
-    or $ref =~ m{#[A-Za-z][-A-Za-z0-9.:_]*$}  # plain-name fragment
-    or $ref =~ m{#/(?:[^~]|~[01])*$};         # json pointer fragment
-  E($state, '%s value is not a valid URI reference', $state->{keyword});
+
+  return E($state, '%s value is not a valid URI reference', $state->{keyword})
+    # see also uri-reference format sub
+    if fc(Mojo::URL->new($ref)->to_unsafe_string) ne fc($ref)
+      or $ref =~ /[^[:ascii:]]/
+      or $ref =~ /#/
+        and $ref !~ m{#$}                          # empty fragment
+        and $ref !~ m{#[A-Za-z][-A-Za-z0-9.:_]*$}  # plain-name fragment
+        and $ref !~ m{#/(?:[^~]|~[01])*$};         # json pointer fragment
+
+  return 1;
+}
+
+sub assert_uri {
+  my ($state, $schema, $override) = @_;
+
+  my $string = $override // $schema->{$state->{keyword}};
+  my $uri = Mojo::URL->new($string);
+
+  return E($state, '"%s" is not a valid URI', $string)
+    # see also uri format sub
+    if fc($uri->to_unsafe_string) ne fc($string)
+      or $string =~ /[^[:ascii:]]/
+      or not $uri->is_abs
+      or $string =~ /#/
+        and $string !~ m{#$}                          # empty fragment
+        and $string !~ m{#[A-Za-z][-A-Za-z0-9.:_]*$}  # plain-name fragment
+        and $string !~ m{#/(?:[^~]|~[01])*$};         # json pointer fragment
+
+  return 1;
 }
 
 # produces an annotation whose value is the same as that of the current keyword
@@ -276,7 +304,7 @@ JSON::Schema::Draft201909::Utilities - Internal utilities for JSON::Schema::Draf
 
 =head1 VERSION
 
-version 0.024
+version 0.025
 
 =head1 SYNOPSIS
 
@@ -287,7 +315,8 @@ version 0.024
 This class contains internal utilities to be used by L<JSON::Schema::Draft201909>.
 
 =for Pod::Coverage is_type get_type is_equal is_elements_unique jsonp local_annotations
-canonical_schema_uri E A abort assert_keyword_type assert_pattern assert_uri_reference annotate_self
+canonical_schema_uri E A abort assert_keyword_type assert_pattern assert_uri_reference assert_uri
+annotate_self
 
 =head1 SUPPORT
 
